@@ -15,6 +15,8 @@
  */
 
 #include <pthread.h>
+#include <stddef.h>
+#include <qb/qblist.h>
 #include "libknet.h"
 #include "onwire.h"
 #include "compat.h"
@@ -155,6 +157,8 @@ struct knet_fd_trackers {
 
 #define KNET_MAX_COMPRESS_METHODS UINT8_MAX
 
+#define KNET_MAX_CRYPTO_INSTANCES 2
+
 struct knet_handle_stats_extra {
 	uint64_t tx_crypt_pmtu_packets;
 	uint64_t tx_crypt_pmtu_reply_packets;
@@ -213,7 +217,9 @@ struct knet_handle {
 	int pmtud_running;
 	int pmtud_forcerun;
 	int pmtud_abort;
-	struct crypto_instance *crypto_instance;
+	struct crypto_instance *crypto_instance[KNET_MAX_CRYPTO_INSTANCES + 1]; /* store an extra pointer to allow 0|1|2 values without too much magic in the code */
+	uint8_t crypto_in_use_config;		/* crypto config to use for TX */
+	uint8_t crypto_only;			/* allow only crypto (1) or also clear (0) traffic */
 	size_t sec_block_size;
 	size_t sec_hash_size;
 	size_t sec_salt_size;
@@ -399,182 +405,18 @@ typedef struct knet_transport_ops {
  * and fd_tracker read lock (from RX thread)
  */
 	int (*transport_rx_is_data)(knet_handle_t knet_h, int sockfd, struct knet_mmsghdr *msg);
-} knet_transport_ops_t;
 
-socklen_t sockaddr_len(const struct sockaddr_storage *ss);
+/*
+ * this function is called by links.c when a link down event is recorded
+ * to notify the transport that packets are not going through, and give
+ * transport the opportunity to take actions.
+ */
+	int (*transport_link_is_down)(knet_handle_t knet_h, struct knet_link *link);
+} knet_transport_ops_t;
 
 struct pretty_names {
 	const char *name;
 	uint8_t val;
 };
 
-/**
- * This is a kernel style list implementation.
- *
- * @author Steven Dake <sdake@redhat.com>
- */
-
-struct knet_list_head {
-	struct knet_list_head *next;
-	struct knet_list_head *prev;
-};
-
-/**
- * @def KNET_LIST_DECLARE()
- * Declare and initialize a list head.
- */
-#define KNET_LIST_DECLARE(name) \
-    struct knet_list_head name = { &(name), &(name) }
-
-#define KNET_INIT_LIST_HEAD(ptr) do { \
-	(ptr)->next = (ptr); (ptr)->prev = (ptr); \
-} while (0)
-
-/**
- * Initialize the list entry.
- *
- * Points next and prev pointers to head.
- * @param head pointer to the list head
- */
-static inline void knet_list_init(struct knet_list_head *head)
-{
-	head->next = head;
-	head->prev = head;
-}
-
-/**
- * Add this element to the list.
- *
- * @param element the new element to insert.
- * @param head pointer to the list head
- */
-static inline void knet_list_add(struct knet_list_head *element,
-			       struct knet_list_head *head)
-{
-	head->next->prev = element;
-	element->next = head->next;
-	element->prev = head;
-	head->next = element;
-}
-
-/**
- * Add to the list (but at the end of the list).
- *
- * @param element pointer to the element to add
- * @param head pointer to the list head
- * @see knet_list_add()
- */
-static inline void knet_list_add_tail(struct knet_list_head *element,
-				    struct knet_list_head *head)
-{
-	head->prev->next = element;
-	element->next = head;
-	element->prev = head->prev;
-	head->prev = element;
-}
-
-/**
- * Delete an entry from the list.
- *
- * @param _remove the list item to remove
- */
-static inline void knet_list_del(struct knet_list_head *_remove)
-{
-	_remove->next->prev = _remove->prev;
-	_remove->prev->next = _remove->next;
-}
-
-/**
- * Replace old entry by new one
- * @param old: the element to be replaced
- * @param new: the new element to insert
- */
-static inline void knet_list_replace(struct knet_list_head *old,
-		struct knet_list_head *new)
-{
-	new->next = old->next;
-	new->next->prev = new;
-	new->prev = old->prev;
-	new->prev->next = new;
-}
-
-/**
- * Tests whether list is the last entry in list head
- * @param list: the entry to test
- * @param head: the head of the list
- * @return boolean true/false
- */
-static inline int knet_list_is_last(const struct knet_list_head *list,
-		const struct knet_list_head *head)
-{
-	return list->next == head;
-}
-
-/**
- * A quick test to see if the list is empty (pointing to it's self).
- * @param head pointer to the list head
- * @return boolean true/false
- */
-static inline int32_t knet_list_empty(const struct knet_list_head *head)
-{
-	return head->next == head;
-}
-
-
-/**
- * Get the struct for this entry
- * @param ptr:	the &struct list_head pointer.
- * @param type:	the type of the struct this is embedded in.
- * @param member:	the name of the list_struct within the struct.
- */
-#define knet_list_entry(ptr,type,member)\
-	((type *)((char *)(ptr)-(char*)(&((type *)0)->member)))
-
-/**
- * Get the first element from a list
- * @param ptr:	the &struct list_head pointer.
- * @param type:	the type of the struct this is embedded in.
- * @param member:	the name of the list_struct within the struct.
- */
-#define knet_list_first_entry(ptr, type, member) \
-	knet_list_entry((ptr)->next, type, member)
-
-/**
- * Iterate over a list
- * @param pos:	the &struct list_head to use as a loop counter.
- * @param head:	the head for your list.
- */
-#define knet_list_for_each(pos, head) \
-	for (pos = (head)->next; pos != (head); pos = pos->next)
-
-/**
- * Iterate over a list backwards
- * @param pos:	the &struct list_head to use as a loop counter.
- * @param head:	the head for your list.
- */
-#define knet_list_for_each_reverse(pos, head) \
-	for (pos = (head)->prev; pos != (head); pos = pos->prev)
-
-/**
- * Iterate over a list safe against removal of list entry
- * @param pos:	the &struct list_head to use as a loop counter.
- * @param n:		another &struct list_head to use as temporary storage
- * @param head:	the head for your list.
- */
-#define knet_list_for_each_safe(pos, n, head) \
-	for (pos = (head)->next, n = pos->next; pos != (head); \
-		pos = n, n = pos->next)
-
-/**
- * Iterate over list of given type
- * @param pos:	the type * to use as a loop counter.
- * @param head:	the head for your list.
- * @param member:	the name of the list_struct within the struct.
- */
-#define knet_list_for_each_entry(pos, head, member)			\
-	for (pos = knet_list_entry((head)->next, typeof(*pos), member);	\
-	     &pos->member != (head);					\
-	     pos = knet_list_entry(pos->member.next, typeof(*pos), member))
-
-
 #endif
